// METATRADER 4 EXPERT ADVISOR
// NAME: Heiken-Ashi_&_Fisher-Transform__EA.mq4
// AUTHOR: Brian M. Lillard, March, 28. 2013
// QUOTE: "The_program_needs_programming_and_the_programming_needs_programming" -Brian.
//+------------------------------------------------------------------+
//--OBJECTIVE: Place either a "New trade-order", or, place a "Signal to signl trade-order".
//--THEORY: Good indicator confirmations for sizable profits'.
//---1. Heiken-Ashi & Fisher-Transform signals must match to open an order.
//---2. Orders are closed when there is a Stoploss, Takeprofit or a New signal.
//--APPLICATIONS: Any Currency, Any Timeframe and either ECN or STP Brokers.
//--DEBUG: Print to "Experts tab" Digits, Spread, SpreadPips, and Ticks. Also Tick_Timeout.
//--EASE OF USE: Easily change strategies, or order settings, and opened orders are u&da@ed.
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Brian M. Lillard"
#include <IBFX - Quick Scripts.mqh>//C:\IBFX_0-Click\experts\include\IBFX - Quick Scripts.mqh
//+------------------------------------------------------------------+
#define BUY   0
#define SELL  1
#define NA    2
//+------------------------------------------------------------------+
extern   string Note1="= Heiken Ashi & Fisher Transform Expert Advisor =";
extern      int Magic=4, Timeframe=60; 
extern     bool SignalToSignal=True;
extern     bool Hedge=False;
extern   string HeikenAshi="= Heiken Ashi Indicator =";  //Heiken_Ashi_Ma_20.mq4
extern      int MaPeriod=20, MaMethod=1, HAbar=1;
extern   string Fisher="= Fisher Transform Indicator ="; //fisher.mq4
extern      int Length=6, Price=0, NumBars=0, FTbar=1;
extern   string Note2="= User Settings =";
extern   double Lot=0.10;
extern      int TrailingStart=30; //10=stop loss at 10 when at 25, and increases 1 pip at a time.
extern      int TrailingStop=5;  //25=stoploss modified to 10 and the price determines if stoploss is needed.
extern      int Loss=35, Profit=150, Retries=3, Slippage=3; 
extern   double Spread=3.0;
extern   string Note3="= Debug Options =";
extern      int tick_timeout=0; 
extern     bool ShowComments=True, DrawEntryBars=True, DrawArrows=True,
                SoundOn?=True, TickSoundOn?=False, TrailSoundOn?=True, CloseSoundOn?=True,
                PrintDigits=False, PrintSpread=False, PrintBars=False, PrintTicks=False, PrintTrailingStartStop=True;
//+------------------------------------------------------------------+
         string Order_Comment;
            int getHA_Value, getFisherTransform_Value, Signl = NA, myBars = 0, barCnt = 0, tick, signls = -1;    
         double haOpen0 = 0, haClose0 = 0, haOpen1 = 0, haClose1 = 0, fClose0 = 0, fClose1 = 0, Spreadpips;
//+------------------------------------------------------------------+
// Initialize
//+------------------------------------------------------------------+
int init() { 
   tick = tick_timeout; myBars = Bars; 
   Order_Comment="[HAFT("+Timeframe+")]["+Symbol()+"(#"+Magic+")]"+"[TP("+Profit+")][SL("+Loss+")]"; 
   return(0); }
//+------------------------------------------------------------------+
// De-Initialize
//+------------------------------------------------------------------+
int deinit() { if(signls >= 0) { 
   for(int GoCnt=0; GoCnt<signls; GoCnt++) { ObjectDelete("VBar"+signls); } } 
   Comment(""); 
   return(0); }
//+------------------------------------------------------------------+
// Start
//+------------------------------------------------------------------+
int start() { 
   static datetime Bar1Time;
   string Signl.str;
   //-- Debug
   if(tick == tick_timeout) {
      if(SoundOn? && TickSoundOn?){PlaySound("tick.wav");}
      tick = 1; if(PrintTicks){Print("Tick = "+tick);} //"Tick = 1"
      if(PrintDigits){Print(Symbol() + " Digits = " + Digits);} //"EURUSD Digits = 5"
      //-- Spread
      Spreadpips = Ask - Bid; //"0.00017"
      if(Digits == 3){Spreadpips = Spreadpips * 100;}else{if(Digits==5) Spreadpips = Spreadpips * 10000;}
      if(Spreadpips > Spread) {Print("Spread too high to open order. Spread is " + Spreadpips); return(0);}
      else{if(PrintSpread){Print(Symbol() + " Spread = " + DoubleToStr(Spreadpips, 1));}} //"EURUSD Spread = 1.7"
      }
   else{tick++; if(PrintTicks){Print("Tick..."+tick);}}
   //-- Get Signals
   if(Bar1Time != Time[1]) {getHAValue(); getFisherTransformValue(); Bar1Time = Time[1];}
   //-- Comments
   if(ShowComments) {
      if(Signl == BUY) {Signl.str = "BUY";}
      if(Signl == SELL) {Signl.str = "SELL";}
      if(Signl == NA) {Signl.str = "N/A";}
      Comment("[Heiken Ashi & Fisher Transform EA] "+Order_Comment+
      "\n"+"[Heiken Ashi] "+getHA_Value+
      "\n"+"[Fisher Transform] "+getFisherTransform_Value+
      "\n"+"[SPREAD] "+DoubleToStr(Spreadpips,1)+"//"+DoubleToStr(Spread,1)+
      "\n"+"[SIGNAL] "+Signl.str);}
   else{Comment("");}
   //-- Trailing Start & Trailing Stop or Execute Order
   for(int i = OrdersTotal() - 1; i >= 0; i--) {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==True) {
         if(OrderSymbol() == Symbol() && OrderType() == OP_BUY && OrderMagicNumber() == Magic) { 
            TrailOrder(OP_BUY);
            if(!SignalToSignal){return(0);} }
         if(OrderSymbol() == Symbol() && OrderType() == OP_SELL && OrderMagicNumber() == Magic) { 
            TrailOrder(OP_SELL);
            if(!SignalToSignal){return(0);} }
         }
      else {
         Print("Could not select the trade to trail order");
         } 
      }
   //-- Bars Only Execution
   if(myBars!=Bars){myBars=Bars;}else{return(0);}
   if(PrintBars){Print("Bar "+ barCnt);}
   //-- EXECUTE: OPEN ORDER, MODIFY ORDER, OR CLOSE ORDER.
   if(getHA_Value == BUY && getFisherTransform_Value == BUY && Signl != BUY && CountOrders(OP_BUY) <= 0 && if(CloseSoundOn?){PlaySound("Alert.wav");})
      if(!Hedge){CloseAll(Symbol(), Magic, OP_SELL, Retries, Slippage, Order_Comment);}    
      Signl = BUY;
      signls++;
      EnterLong(Symbol(), Lot, Order_Comment, Loss, Profit, Magic, Retries, Slippage);
      if(SoundOn?){PlaySound("ok.wav");}
      if(DrawArrows){SetArrow(Time[0], Ask, 241, Red);}
      if(DrawEntryBars){CreateVLine("VBar" + signls, true);}
      }
   if(getHA_Value == SELL && getFisherTransform_Value == SELL && Signl != SELL && CountOrders(OP_SELL) <= 0 && if(CloseSoundOn?){PlaySound("Alert.wav");})
      if(!Hedge){CloseAll(Symbol(), Magic, OP_BUY, Retries, Slippage, Order_Comment);}
      Signl = SELL;
      signls++;
      EnterShrt(Symbol(), Lot, Order_Comment, Loss, Profit, Magic, Retries, Slippage);
      if(SoundOn?){PlaySound("ok.wav");}
      if(DrawArrows){SetArrow(Time[0], Bid, 242, Blue);}
      if(DrawEntryBars){CreateVLine("VBar" + signls, false);}
      } 
   //-- Return Start
   return(0); 
   }
//+------------------------------------------------------------------+
//Count Orders
//+------------------------------------------------------------------+
int CountOrders(int direction) { 
   int j=0;                                                            
   for(int i=OrdersTotal()-1; i>=0; i--)
      {//   
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES)==True)
         {//          
         if(OrderType() == direction && OrderSymbol() == Symbol() && OrderMagicNumber() == Magic)
            {
            j++; 
            } 
         }
      else
         {
         Print("Could not SELECT trade"); 
         } 
      }
   return(j); 
   }
//+------------------------------------------------------------------+
//Get Heiken Ashi Signals
//+------------------------------------------------------------------+
void getHAValue() { 
   getHA_Value = NA;
   haOpen0=iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,2,HAbar);
   haClose0= iCustom(NULL,Timeframe,"Heiken_Ashi_Ma_20",MaMethod,MaPeriod,3,HAbar);  
   if(haOpen0 < haClose0) {
      getHA_Value = BUY;//buy
      }
   if(haOpen0 > haClose0) {
      getHA_Value = SELL;//sell
      }
   }
//+------------------------------------------------------------------+
//Get Fisher Transform Signals
//+------------------------------------------------------------------+
void getFisherTransformValue() { 
   getFisherTransform_Value = NA;
   fClose0 = iCustom(NULL, Timeframe, "fisher", Length, Price, NumBars, 0, FTbar);
   fClose1 = iCustom(NULL, Timeframe, "fisher", Length, Price, NumBars, 1, FTbar);
   if(fClose0 < fClose1) {getFisherTransform_Value = BUY;} //buy
   if(fClose0 > fClose1) {getFisherTransform_Value = SELL;} //sell
   }
//+------------------------------------------------------------------+
// Trailing Start & Trailing Stop
//+------------------------------------------------------------------+
void TrailOrder(int type) { 
   for(int h; barCnt=OrdersTotal()-1; h>=0; h--) {
      if(OrderSelect(h,SELECT_BY_POS,MODE_TRADES) == True && TrailingStart > 0 || TrailingStop > 0) {
         if(OrderMagicNumber() == Magic && OrderComment() == Order_Comment) {
            if(type == OP_BUY) {
               double sl = NormalizeDouble(OrderStopLoss()-OrderOpenPrice(),Digits);                   
               double profit=NormalizeDouble(Bid-OrderOpenPrice(),Digits);                        
               if(profit>=Point*TrailingStart+Point*TrailingStop) {//We are over the minimum profit by the trail
                  if(sl<Point*TrailingStart) {//Move SL to minimum profit
                     OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Green);
                     if(PrintTrailingStartStop) {
                        Print("Moved SL to: "+(Bid-Point*TrailingStop));
                        } 
                     if(SoundOn?) {PlaySound("stops.wav");} 
                     return(0);
                     }                      
                  if(profit-sl > Point*TrailingStop) {// Keep moving the Trailing Stop up as we have more profit.
                     OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Green);
                     if(SoundOn?) {PlaySound("stops.wav");} 
                     return(0);
                     } 
                  }
               }
            if(type==OP_SELL) {
               sl = NormalizeDouble(OrderOpenPrice()-OrderStopLoss(),Digits);
               profit = NormalizeDouble(OrderOpenPrice()-Ask,Digits);
               if(profit >= Point * TrailingStart + Point * TrailingStop) { // We are over the minimum profit by the trail
                  if(sl < Point * TrailingStart) { // Move SL to minimum profit
                     OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Red); 
                      Print("Moved SL to min " + (Ask+Point*TrailingStop));
                      if(SoundOn?) {
                        PlaySound("stops.wav");
                        } 
                     return(0);
                     }
                  if(profit-sl > Point*TrailingStop) {// Keep moving the Trailing Stop up as we have more profit.
                     OrderModify(OrderTicket(),OrderOpenPrice(),Ask + Point * TrailingStop,OrderTakeProfit(),0,Red);
                     if(SoundOn?) {
                        PlaySound("stops.wav");
                        } 
                     return(0);
                     } 
                  } 
               }
            } 
         } 
      }
   }
//+------------------------------------------------------------------+
// Set Arrows
//+------------------------------------------------------------------+
void SetArrow(datetime t, double p, int k, color c) { 
    ObjectSet("Arrow", OBJPROP_TIME1 , t);
    ObjectSet("Arrow", OBJPROP_PRICE1 , p);
    ObjectSet("Arrow", OBJPROP_ARROWCODE, k);
    ObjectSet("Arrow", OBJPROP_COLOR , c);
   }
//+------------------------------------------------------------------+
//Draw Vertical Lines
//+------------------------------------------------------------------+
void CreateVLine(string Name, bool buy) { 
   if(buy) { 
      ObjectCreate(Name,OBJ_VLINE,0,Time[0],Ask); 
      ObjectSet(Name,OBJPROP_COLOR,Red);
      ObjectSet(Name,OBJPROP_STYLE,3); 
      ObjectSet(Name,OBJPROP_WIDTH,0); ObjectSet(Name,OBJPROP_BACK,false); 
      }
   else { 
      ObjectCreate(Name,OBJ_VLINE,0,Time[0],Bid); 
      ObjectSet(Name,OBJPROP_COLOR,RoyalBlue);
      ObjectSet(Name,OBJPROP_STYLE,3); 
      ObjectSet(Name,OBJPROP_WIDTH,0); 
      ObjectSet(Name,OBJPROP_BACK,false); 
      }
   }

